@@ -69,7 +69,7 @@
  * Paul Mackerras (paulus@cs.anu.edu.au).
  */
 
-/* $Id: if_ppp.c,v 1.10 1994/04/25 06:56:55 paulus Exp $ */
+/* $Id: if_ppp.c,v 1.12 1994/05/24 23:00:05 paulus Exp $ */
 /* from if_sl.c,v 1.11 84/10/04 12:54:47 rick Exp */
 
 #include "ppp.h"
@@ -88,6 +88,7 @@
 #include "tty.h"
 #include "kernel.h"
 #include "conf.h"
+#include "vnode.h"
 
 #include "if.h"
 #include "if_types.h"
@@ -376,7 +377,7 @@ pppread(tp, uio, flag)
 	return 0;
     s = splimp();
     while (sc->sc_inq.ifq_head == NULL && tp->t_line == PPPDISC) {
-	if (tp->t_state & TS_ASYNC) {
+	if (tp->t_state & TS_ASYNC || flag & IO_NDELAY) {
 	    splx(s);
 	    return (EWOULDBLOCK);
 	}
@@ -661,7 +662,7 @@ pppoutput(ifp, m0, dst)
 	 * put the packet on the fastq instead.
 	 */
 	if ((ip = mtod(m0, struct ip *))->ip_p == IPPROTO_TCP) {
-	    register int p = ((int *)ip)[ip->ip_hl];
+	    register int p = ntohl(((int *)ip)[ip->ip_hl]);
 	    if (INTERACTIVE(p & 0xffff) || INTERACTIVE(p >> 16))
 		ifq = &sc->sc_fastq;
 	}
@@ -1371,16 +1372,15 @@ pppinput(c, tp)
 	return;
     }
 
-    if (c == PPP_ESCAPE) {
-	sc->sc_flags |= SC_ESCAPED;
-	return;
-    }
     if (c < 0x20 && (sc->sc_rasyncmap & (1 << c)))
 	return;
 
     if (sc->sc_flags & SC_ESCAPED) {
 	sc->sc_flags &= ~SC_ESCAPED;
 	c ^= PPP_TRANS;
+    } else if (c == PPP_ESCAPE) {
+	sc->sc_flags |= SC_ESCAPED;
+	return;
     }
 
     /*
