@@ -88,6 +88,7 @@
 #include <sys/tty.h>
 #include <sys/kernel.h>
 #include <sys/conf.h>
+#include <sys/vnode.h>
 
 #include <net/if.h>
 #include <net/if_types.h>
@@ -363,7 +364,7 @@ pppread(tp, uio, flag)
 	return 0;
     s = splimp();
     while (sc->sc_inq.ifq_head == NULL && tp->t_line == PPPDISC) {
-	if (tp->t_state & TS_ASYNC) {
+	if (tp->t_state & TS_ASYNC || flag & IO_NDELAY) {
 	    splx(s);
 	    return (EWOULDBLOCK);
 	}
@@ -648,7 +649,7 @@ pppoutput(ifp, m0, dst)
 	 * put the packet on the fastq instead.
 	 */
 	if ((ip = mtod(m0, struct ip *))->ip_p == IPPROTO_TCP) {
-	    register int p = ((int *)ip)[ip->ip_hl];
+	    register int p = ntohl(((int *)ip)[ip->ip_hl]);
 	    if (INTERACTIVE(p & 0xffff) || INTERACTIVE(p >> 16))
 		ifq = &sc->sc_fastq;
 	}
@@ -1365,16 +1366,15 @@ pppinput(c, tp)
 	return;
     }
 
-    if (c == PPP_ESCAPE) {
-	sc->sc_flags |= SC_ESCAPED;
-	return;
-    }
     if (c < 0x20 && (sc->sc_rasyncmap & (1 << c)))
 	return;
 
     if (sc->sc_flags & SC_ESCAPED) {
 	sc->sc_flags &= ~SC_ESCAPED;
 	c ^= PPP_TRANS;
+    } else if (c == PPP_ESCAPE) {
+	sc->sc_flags |= SC_ESCAPED;
+	return;
     }
 
     /*
