@@ -6,7 +6,7 @@
   Copyright (C) 1990  Brad K. Clements, All Rights Reserved
   See copyright notice in NOTES
 
-  $Id: ppp_if.c,v 1.7 1994/04/21 03:31:21 paulus Exp $
+  $Id: ppp_if.c,v 1.8 1994/05/27 00:40:23 paulus Exp $
 */
 
 #define	VJC	1
@@ -54,7 +54,6 @@
 #include <net/slcompress.h>
 #endif
 
-#include <net/if_ppp.h>
 #include <net/ppp_str.h>
 
 #define MAX_PKTSIZE	4096	/* max packet size including framing */
@@ -500,7 +499,6 @@ ppp_if_rsrv(q)
     int dlen;
 #endif
     register PII *p;
-    struct ppp_header *ph;
     struct mbuf	*mb1, *mb2,*mbtail;
     int	len, xlen, count, s;
     u_char *rptr;
@@ -526,10 +524,9 @@ ppp_if_rsrv(q)
 	    continue;
 	}
 	m0 = mp;	/* remember first message block */
-	ph = (struct ppp_header *) mp->b_rptr;
 
 #ifdef	VJC
-	switch (ntohs(ph->ph_protocol)) {
+	switch (PPP_PROTOCOL(mp->b_rptr)) {
 	case PPP_VJC_COMP :
 	    if (p->pii_flags & PII_FLAGS_VJC_REJ) {
 		DLOG("VJC rejected\n", 0);
@@ -572,8 +569,7 @@ ppp_if_rsrv(q)
 	     * Set up the protocol field, in case this message
 	     * gets put back on the queue.
 	     */
-	    ph = (struct ppp_header *) mp->b_rptr;
-	    ph->ph_protocol = htons(PPP_IP);
+	    mp->b_rptr[3] = PPP_IP;
 	    DLOG("ppp_if%d: VJC decompressed\n", p - pii);
 	    break;
 
@@ -596,13 +592,13 @@ ppp_if_rsrv(q)
 	     * Set up the protocol field, in case this message
 	     * gets put back on the queue.
 	     */
-	    ph->ph_protocol = htons(PPP_IP);
+	    mp->b_rptr[3] = PPP_IP;
 	    DLOG("ppp_if%d: VJU decompressed\n", p - pii);
 	    break;
 	}
 #endif
 
-	switch (ntohs(ph->ph_protocol)) {
+	switch (PPP_PROTOCOL(mp->b_rptr)) {
 	default:
 	    if (!canput(q->q_next)) {
 		putbq(q, mp);
